package com.zenika.presentation.annotations.compilation.serializable;

import java.io.Serializable;
import java.util.List;
import java.util.Set;
import javax.annotation.processing.AbstractProcessor;
import javax.annotation.processing.Messager;
import javax.annotation.processing.RoundEnvironment;
import javax.annotation.processing.SupportedAnnotationTypes;
import javax.annotation.processing.SupportedSourceVersion;
import javax.lang.model.SourceVersion;
import javax.lang.model.element.Element;
import javax.lang.model.element.ElementKind;
import javax.lang.model.element.TypeElement;
import javax.lang.model.type.TypeMirror;
import javax.lang.model.util.Elements;
import javax.lang.model.util.Types;
import javax.tools.Diagnostic;

@SupportedSourceVersion(SourceVersion.RELEASE_6)
@SupportedAnnotationTypes("com.zenika.presentation.annotations.compilation.serializable.SerializableClasses")
public class SerializableClassesProcessor extends AbstractProcessor {
    @Override
    public boolean process(Set<? extends TypeElement> annotations, RoundEnvironment roundEnv) {
        // Utils
        Types typeUtils = processingEnv.getTypeUtils();
        Elements elementUtils = processingEnv.getElementUtils();
        Messager messager = processingEnv.getMessager();

        // The Serializable interface - used for comparison
        // We can do this because Serializable is a core Java class, therefore always loaded
        TypeMirror serializable = elementUtils.getTypeElement(Serializable.class.getCanonicalName()).asType();

        // Find the annotated packages
        for (TypeElement annotation : annotations) {
            for (Element pkg : roundEnv.getElementsAnnotatedWith(annotation)) {

                // List the elements inside the package
                List<? extends Element> pkgElements = pkg.getEnclosedElements();
                for (Element pkgElement : pkgElements) {

                    // Keep classes only (not enums or interfaces)
                    if (pkgElement.getKind() != ElementKind.CLASS) {
                        continue;
                    }

                    // Test if the class is Serializable
                    boolean isSerializable = typeUtils.isAssignable(pkgElement.asType(), serializable);
                    if (!isSerializable) {
                        messager.printMessage(Diagnostic.Kind.ERROR,
                                "The following class is not Serializable : " + pkg.getSimpleName() + "." + pkgElement.getSimpleName());
                    }

                }

            }
        }

        // Prevent other processors from processing this annotation
        return true;

    }

}